-- Lightroom SDK
local LrView = import "LrView"
local bind = LrView.bind
local LrPathUtils = import "LrPathUtils"
local LrTasks = import "LrTasks"
local LrLogger = import "LrLogger"
local log = LrLogger("MergeTitles")
local LrDialogs = import "LrDialogs"
local LrPrefs = import "LrPrefs"
require("UpdateCheck.lua")

--============================================================================--

local MergeTitles = {}

function MergeTitles.sectionForFilterInDialog( viewFactory, propertyTable )
	local prefs = LrPrefs.prefsForPlugin()

	if propertyTable.CopyMode == nil then
		propertyTable.CopyMode = prefs.CopyMode
	end
	if propertyTable.NeverReplace == nil then
		propertyTable.NeverReplace = prefs.NeverReplace
	end

	return {
		title = "Merge Titles",
		
		viewFactory:row {
			spacing = viewFactory:control_spacing(),
			
			viewFactory:row {
				viewFactory:static_text {
					title = "Copy",
				},

				viewFactory:popup_menu {
					value = LrView.bind("CopyMode"),
					width_in_chars = 32,
					items = {
						{ title = "Caption to Title", value = 0 },
						{ title = "Title to Caption", value = 1 },
						{ title = "Either if the other is unset", value = 2 },
					},
				},
			},
			
			viewFactory:row {
				spacing = viewFactory:control_spacing(),
				
				viewFactory:checkbox {
					title = "Never replace a value",
					value = LrView.bind("NeverReplace"),
				},
			},
		},
	}
end

function MergeTitles.DoPhotoMerge( thePhoto, prefs )
	local titleField
	local captionField
	local changed = false
	
	titleField = thePhoto:getFormattedMetadata("title") or ""
	captionField = thePhoto:getFormattedMetadata("caption") or ""
	
    log:info("- Caption: '" .. captionField .. "'")
    log:info("- Title: '" .. captionField .. "'")

	if captionField ~= titleField then
		if prefs.CopyMode == 0 then -- Caption to Title
			if titleField == "" or not prefs.NeverReplace then
				thePhoto:setRawMetadata("title", captionField)
				changed = true
			end
		elseif prefs.CopyMode == 1 then -- Title to Caption
			if captionField == "" or not prefs.NeverReplace then
				thePhoto:setRawMetadata("caption", titleField)
				changed = true
			end
		else -- Either, if one is empty
			if titleField ~= "" and captionField == "" then
				thePhoto:setRawMetadata("caption", titleField)
				changed = true
			elseif captionField ~= "" and titleField == "" then
				thePhoto:setRawMetadata("title", captionField)
				changed = true
			end
		end
	end
	
	return changed
end

-------------------------------------------------------------------------------

function MergeTitles.postProcessRenderedPhotos( functionContext, filterContext )
	Count = 0
	log:enable("logfile")
	
	local renditionOptions = {
		plugin = _PLUGIN,
		renditionsToSatisfy = filterContext.renditionsToSatisfy,
		filterSettings = function( renditionToSatisfy, exportSettings )
			local dims, odims
			local prefs = LrPrefs.prefsForPlugin()

            log:info("Replace Mode: " .. prefs.CopyMode)
            log:info("Never Replace: " .. (prefs.NeverReplace and "True" or "False"))

			if UpdateCheck.StringLookup(prefs) >= 1 or Count < 3 then
				renditionToSatisfy.photo.catalog:withWriteAccessDo("Merge Titles", function()
					if MergeTitles.DoPhotoMerge(renditionToSatisfy.photo, prefs) then
						Count = Count + 1
					end
				end)
			end

		end,
	}
	
	log:enable("logfile")
	log:trace("Starting export")

	for sourceRendition, renditionToSatisfy in filterContext:renditions(renditionOptions) do
		local success, pathOrMessage = sourceRendition:waitForRender()
	end
end

--------------------------------------------------------------------------------

MergeTitles.exportPresetFields = {
    { key = "NeverReplace", default = true },
    { key = "CopyMode", default = 2 },
}

return MergeTitles
